﻿using Corsinvest.ProxmoxVE.Api;
using Microsoft.Extensions.Configuration;
using Newtonsoft.Json;
using ProxmoxConnector.Integration.Abstractions;
using System;
using System.Threading.Tasks;

namespace ProxmoxConnector.Integration.Corsinvest.Services
{
    internal class PveConnector : IPveConnector
    {
        private readonly string _hostName;
        private readonly int _port;
        private readonly string _userName;
        private readonly string _password;
        private readonly string _token;

        public PveConnector(IConfiguration configuration)
        {
            var config = configuration.GetSection("Pve");
            _hostName = config["HostName"];
            _port = config.GetValue<int>("Port");
            _userName = config["UserName"];
            _password = config["Password"];
            _token = config["Token"];
        }

        public async Task TestWithLogin()
        {
            var client = new PveClient(_hostName, _port);
            if (await client.Login(_userName, _password))
            {
                var vm = client.Nodes["pve-lora"].Qemu[100];

                //config vm
                var config = await vm.Config.VmConfig();
                Console.WriteLine(JsonConvert.SerializeObject(config.Response, Formatting.Indented));

                //create snapshot
                var response = await vm.Snapshot.Snapshot("pippo2311");

                //update snapshot description
                await vm.Snapshot["pippo2311"].Config.UpdateSnapshotConfig("description");

                //delete snapshot
                await vm.Snapshot["pippo2311"].Delsnapshot();

                //list of snapshot
                foreach (var snapshot in (await vm.Snapshot.SnapshotList()).Response.data)
                {
                    Console.WriteLine(JsonConvert.SerializeObject(snapshot, Formatting.Indented));
                    Console.WriteLine(snapshot.name);
                }

                //change response type from json to png
                client.ResponseType = ResponseType.Png;
                var dataImg = (await client.Nodes["pve-test1"].Rrd.Rrd("cpu", "day")).Response;
                Console.WriteLine($"<img src=\"{dataImg}\" \\>");
            }
        }

        public async Task TestWithToken()
        {
            //using Api Token
            var client2 = new PveClient(_hostName, _port);
            client2.ApiToken = _token;
            var version = await client2.Version.Version();
            Console.WriteLine(JsonConvert.SerializeObject(version.Response.data, Formatting.Indented));
        }

        public async Task Start()
        {
            var client = new PveClient(_hostName, _port);
            if (await client.Login(_userName, _password))
            {
                var vm = client.Nodes["pve-lora"].Qemu[100];

                var startEvent = vm.Status.Start;
                var start = await startEvent.VmStart();
                if (start.IsSuccessStatusCode)
                    Console.WriteLine(JsonConvert.SerializeObject(start.Response, Formatting.Indented));
            }
        }

        public async Task Stop()
        {
            var client = new PveClient(_hostName, _port);
            if (await client.Login(_userName, _password))
            {
                var vm = client.Nodes["pve-lora"].Qemu[100];

                var stopEvent = vm.Status.Stop;
                var stop = await stopEvent.VmStop();
                if (stop.IsSuccessStatusCode)
                    Console.WriteLine(JsonConvert.SerializeObject(stop.Response, Formatting.Indented));
            }
        }
    }
}
